SystemOrganization addCategory: #'CS528-Project'!
SystemOrganization addCategory: #'CS528-Project-Seaside'!
SystemOrganization addCategory: #'CS528-Project-Websocket'!

ZnWebSocket subclass: #TableWebsocket
	instanceVariableNames: 'tableID delegate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Websocket'!

!TableWebsocket commentStamp: '<historical>' prior: 0!
A TableWebsocket is an abstraction over the websocket connection between a PokerPlayer and the PokerTable they are currently a member of. It is a reference maintained by the PokerPlayer so that 

Instance Variables
	delegate:		<PokerPlayer>
	tableID:		<string>

delegate
	- PokerPlayer which will be required to handle incoming messages on this websocket. Must implement onMessage: message.

tableID
	- string representing the ID of the table this websocket is connected to.
!

!TableWebsocket class methodsFor: 'initialize' stamp: 'BrianDentino 7/11/2013 21:45'!
tableID: id delegate: aDelegate 
	^ ((TableWebsocket to: 'ws://localhost:', ZnServer default port asString, '/', id) tableID: id) delegate: aDelegate.! !

!TableWebsocket methodsFor: 'initialize-release' stamp: 'lb 6/13/2013 21:46'!
delegate: aDelegate
	delegate := aDelegate.
	[[self runWith: [ :message |
      	self crLog: 'PokerPlayer received message from table: ', message printString. 
		delegate onMessage: message from: self].] on: ConnectionClosed do: [ self log debug: '==Connection Closed'] ] fork.! !

!TableWebsocket methodsFor: 'as yet unclassified' stamp: 'BrianDentino 7/11/2013 23:08'!
initialize
	super initialize.
	self role: #server.! !

!TableWebsocket methodsFor: 'initialize-release' stamp: 'BrianDentino 7/6/2013 22:24'!
tableID: id
	tableID := id! !

WAComponent subclass: #ActionButton
	instanceVariableNames: 'callback text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Seaside'!

!ActionButton class methodsFor: 'initialize-release' stamp: 'BrianDentino 7/18/2013 03:43'!
with: aString onClick: aCallback 
	^ self new text: aString; onClick: aCallback.! !

!ActionButton methodsFor: 'as yet unclassified' stamp: 'BrianDentino 7/18/2013 03:41'!
onClick: aCallback
	callback := aCallback.! !

!ActionButton methodsFor: 'as yet unclassified' stamp: 'BrianDentino 7/18/2013 03:42'!
renderContentOn: html
	html submitButton 
		onClick: ( html jQuery ajax
			callback: [ callback value ] );
		with: text.! !

!ActionButton methodsFor: 'as yet unclassified' stamp: 'BrianDentino 7/18/2013 03:51'!
text: aString 
	text := aString.! !

WAComponent subclass: #BetSlider
	instanceVariableNames: 'min max step betAmount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Seaside'!

!BetSlider commentStamp: '<historical>' prior: 0!
A BetSlider is a renderable component that presents a slider and label indicating the current value of the slider.

Instance Variables
	betAmount:		<Number> current bet
	max:		<Number> max slider value
	min:		<Number> min slider value
	step:		<Number> slider increment interval

betAmount
	- xxxxx

max
	- xxxxx

min
	- xxxxx

step
	- xxxxx
!

!BetSlider class methodsFor: 'initialize' stamp: 'BrianDentino 7/15/2013 23:08'!
min: minValue max: maxValue step: stepValue
	^ (self new) min: minValue; max: maxValue; step: stepValue.! !

!BetSlider methodsFor: 'accessing' stamp: 'BrianDentino 7/15/2013 22:34'!
betAmount
	^betAmount.! !

!BetSlider methodsFor: 'accessing' stamp: 'BrianDentino 7/16/2013 04:13'!
betAmount: anAmount
	betAmount := anAmount.! !

!BetSlider methodsFor: 'initialize' stamp: 'BrianDentino 7/15/2013 23:12'!
initialize
	super initialize. 
	min := 0.
	max := 0.
	step := 0.
	betAmount := 0.! !

!BetSlider methodsFor: 'accessing' stamp: 'BrianDentino 7/15/2013 23:09'!
max: aValue
	max := aValue! !

!BetSlider methodsFor: 'accessing' stamp: 'BrianDentino 7/16/2013 02:54'!
min: aValue
	min := aValue.
	betAmount := min! !

!BetSlider methodsFor: 'rendering' stamp: 'BrianDentino 7/16/2013 04:17'!
renderContentOn: html
	| labelID sliderID |
	
	labelID := html nextId.
	sliderID := html nextId.
	
	html div 
		id: sliderID;
		script: (html jQuery new slider
			min: min;
			max: max;
			step: step;
			value: betAmount;
			orientation: #vertical;
			onSlide: [ :value | ( html jQuery script: [ :s | s << (s jQuery id: labelID) text: value ] ) ]
			
			"; 
			Not actually using this anymore, but leaving it in as documentation for how to setup ajax callbacks.
			
			onChange: [ :value | ( html jQuery ajax 
				callback: [ :amount | self betAmount: amount asNumber ] value: (value) ) ] ).
			"
			
			).
		
	html space.
	html space.
	html div id: labelID;
		with: betAmount! !

!BetSlider methodsFor: 'accessing' stamp: 'BrianDentino 7/15/2013 23:09'!
step: aValue
	step := aValue! !

WAComponent subclass: #PokerPlayerView
	instanceVariableNames: 'playerID tableID table browser playerList community hand lastPlayerAction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Seaside'!

PokerPlayerView class
	instanceVariableNames: 'nextID'!

!PokerPlayerView commentStamp: '<historical>' prior: 0!
A PokerPlayer is a Seaside component that manages client decisions. When the state of a game changes, the PokerPlayer class notifies the view via a websocket connection to the browser.

Instance Variables
!

PokerPlayerView class
	instanceVariableNames: 'nextID'!

!PokerPlayerView class methodsFor: 'initialize' stamp: 'BrianDentino 7/12/2013 02:13'!
initialize 
	super initialize.
	nextID := 0.! !

!PokerPlayerView class methodsFor: 'as yet unclassified' stamp: 'lb 7/19/2013 16:17'!
nextID
	nextID := nextID + 1.
	^ nextID asString.! !

!PokerPlayerView class methodsFor: 'initialize' stamp: 'lb 6/13/2013 19:18'!
tableID: table
	^(self new tableID: table playerID: PokerPlayerView nextID)! !

!PokerPlayerView methodsFor: 'actions' stamp: 'lb 7/19/2013 22:41'!
call: anAmount

	| action |
	
	action := Dictionary new.
	action at: 'actionType' put: 'call'.
	action at: 'amount' put: anAmount.
	
	self sendPlayerAction: action! !

!PokerPlayerView methodsFor: 'websockets' stamp: 'BrianDentino 7/12/2013 01:47'!
close
	table close.! !

!PokerPlayerView methodsFor: 'websockets' stamp: 'lb 7/19/2013 10:56'!
commTesting

	table sendMessage: (WebsocketMessage id: #commTesting content: '') json.! !

!PokerPlayerView methodsFor: 'actions' stamp: 'lb 7/19/2013 22:38'!
fold 

	| action |
	
	action := Dictionary new.
	action at: 'actionType' put: 'fold'.
	
	self sendPlayerAction: action! !

!PokerPlayerView methodsFor: 'actions' stamp: 'lb 7/19/2013 11:09'!
handTesting

	table sendMessage: (WebsocketMessage id: #handTesting content: '') json.! !

!PokerPlayerView methodsFor: 'initialize-release' stamp: 'lb 7/19/2013 21:22'!
initialize 
	super initialize.
	browser := BrowserWebsocket delegate: self.
	playerList := Players new.
	community := Hand new.
	hand := Hand new.
	lastPlayerAction := PokerPlayerAction new.! !

!PokerPlayerView methodsFor: 'websockets' stamp: 'lb 7/19/2013 23:10'!
onMessage: aMessage from: websocket

	| msg |

	(websocket == table) ifTrue: [ 
		msg := WebsocketMessage json: aMessage.
		msg id = #bet ifTrue: [  ].
		browser sendMessage: aMessage ].
	(websocket == browser) ifTrue: [ table sendMessage: aMessage ].! !

!PokerPlayerView methodsFor: 'actions' stamp: 'BrianDentino 7/18/2013 03:19'!
placeBet: anAmount 

	| action |
	
	action := Dictionary new.
	action at: 'actionType' put: 'bet'.
	action at: 'amount' put: anAmount.
	
	self sendPlayerAction: action! !

!PokerPlayerView methodsFor: 'accessing' stamp: 'lb 7/13/2013 15:16'!
playerID
	^ playerID! !

!PokerPlayerView methodsFor: 'rendering' stamp: 'lb 7/19/2013 23:15'!
renderContentOn: html

	| betSlider |

	betSlider := BetSlider min: 5 max: 100 step: 5.

	html text: 'Welcome to table ', tableID, '. You are player ', playerID, '.'.
	html horizontalRule.
	
	html render: (ActionButton with: 'CommunityTesting' onClick: [ self commTesting ]).
		html div id: 'community';
		with: [ 
			html render: community.
		].

	html render: (ActionButton with: 'HandTesting' onClick: [ self handTesting ]).
	html horizontalRule.
	
	html render: betSlider.
	html render: (ActionButton with: 'Bet' onClick: [ self placeBet: betSlider betAmount ]).
	html render: (ActionButton with: 'Call' onClick: [ self call: 99 ]). "TODO make this the actual amount..."
	html render: (ActionButton with: 'Fold' onClick: [ self fold ]).
	html div id: 'bet'.
	html horizontalRule.
		
	html text: 'My hand:'.
	html div id: 'hand';
		with: [ 
			html render: hand 
	].
	html horizontalRule.
		
	html div id: 'players';
		with: [ 
			html render: playerList.
		].

	html div id: 'lastPlayerAction';
		with: [ 
			html render: lastPlayerAction.
		].

	"
	html render: (PokerPlayerComponent name: 'brian' money: 500).
	html div id: 'pokerTable';
		with: [ 
			html render: tableComponent.
		]."
	
	"	html render: (ActionButton with: 'Activate' onClick: [ self requestActivation ]).
	html horizontalRule."
	
	html render: (WebsocketHandlerComponent wsAddress: 'ws://localhost:1701/', tableID, '_', playerID).
	
	 self renderJSPlayersUpdateOn: html.! !

!PokerPlayerView methodsFor: 'rendering' stamp: 'BrianDentino 7/18/2013 03:13'!
renderJSPlayersUpdateOn: html
	html script: 
	'
	updatePlayers = function() { 
		var elems = document.getElementsByClassName("pokerPlayer");
		var increase = Math.PI * 2 / elems.length;
		var x = 0, y = 0, angle = 0, elem;

		for (var i = 0; i < elems.length; i++) {
    			elem = elems[i];
    			x = 200 * Math.cos(angle) + 400;
    			y = 200 * Math.sin(angle) + 400;
    			elem.style.position = "absolute";
    			elem.style.left = x + "px";
    			elem.style.top = y + "px";
    			angle += increase;
		}
	}
	'.! !

!PokerPlayerView methodsFor: 'actions' stamp: 'BrianDentino 7/18/2013 03:46'!
requestActivation

	table sendMessage: (WebsocketMessage id: 'activate' content: playerID) json.! !

!PokerPlayerView methodsFor: 'websockets' stamp: 'lb 7/18/2013 01:24'!
sendPlayerAction: anAction

	table sendMessage: (WebsocketMessage id: 'playerAction' content: anAction) json.! !

!PokerPlayerView methodsFor: 'rendering' stamp: 'BrianDentino 7/18/2013 03:55'!
style
	^ '
	div { 
		font-family: monospace;
		text-align: center;
	}
	
	.playerWidget {
    		border: 2px solid #064c0e;   
    		border-radius: 10px;
    		-webkit-border-radius: 10px;
    		-moz-border-radius: 10px;
    
    		width: 120px;
    		padding: 5px 10px;
    
    		background-image: -webkit-gradient(linear, 0 100%, 0 0, color-stop(0.18, rgb(52,82,60)), color-stop(0.74, rgb(104,158,122)));
    		background-image: -webkit-linear-gradient(bottom, rgb(52,82,60) 18%, rgb(104,158,122) 74%);
    		background-image: -moz-linear-gradient(bottom, rgb(52,82,60) 18%, rgb(104,158,122) 74%);
    		background-image: -o-linear-gradient(bottom, rgb(52,82,60) 18%, rgb(104,158,122) 74%);
    		background-image: linear-gradient(bottom, rgb(52,82,60) 18%, rgb(104,158,122) 74%);
	}
	
	.activePlayer {
    		border: 2px solid #400000;
    		background-image: linear-gradient(bottom, rgb(82, 52, 72) 18%, rgb(156, 103, 135) 74%);
    		background-image: -o-linear-gradient(bottom, rgb(82, 52, 72) 18%, rgb(156, 103, 135) 74%);
    		background-image: -moz-linear-gradient(bottom, rgb(82, 52, 72) 18%, rgb(156, 103, 135) 74%);
    		background-image: -webkit-linear-gradient(bottom, rgb(82, 52, 72) 18%, rgb(156, 103, 135) 74%);
    		background-image: -ms-linear-gradient(bottom, rgb(82, 52, 72) 18%, rgb(156, 103, 135) 74%);
    		background-image: -webkit-gradient(linear, left bottom, left top, color-stop(0.18, rgb(82, 52, 72)), color-stop(0.74, rgb(156, 103, 135)));
	}

	.playerName {
    		color : white;
    		font-family: monospace;
    		font-size: 14px;
    		padding: 5px 0px 5px;
    		position: relative;
    		width: 100%;
    		text-align: center;
	}

	.playerMoney {
    		color : #FFE4B5;
    		font-family: monospace;
    		font-size: 18px;
    		padding: 0px 0px 5px;
    		position: relative;
    		width: 96%;
    		text-align: center;
	}'.! !

!PokerPlayerView methodsFor: 'accessing' stamp: 'BrianDentino 7/6/2013 21:29'!
tableID
	^tableID! !

!PokerPlayerView methodsFor: 'accessing' stamp: 'BrianDentino 7/18/2013 01:38'!
tableID:  aTableID playerID: aPlayerID
	| url |
	tableID := aTableID.
	playerID := aPlayerID.
	url := tableID, '_', playerID.
	table := TableWebsocket tableID: tableID delegate: self.
	
	ZnServer default delegate map: url to: (ZnWebSocketDelegate map: url to: browser).! !

WAComponent subclass: #ProjectLanding
	instanceVariableNames: ''
	classVariableNames: 'tables'
	poolDictionaries: ''
	category: 'CS528-Project-Seaside'!

ProjectLanding class
	instanceVariableNames: 'tables'!

!ProjectLanding commentStamp: '<historical>' prior: 0!
A ProjectLanding is a Seaside component that renders a page which allows users to create or join tables for hosting/playing poker games.

Instance Variables
	tables:		<Object>

tables
	- a dictionary of all active tables. key (tableID) maps to PokerTable object.
!

ProjectLanding class
	instanceVariableNames: 'tables'!

!ProjectLanding class methodsFor: 'as yet unclassified' stamp: 'BrianDentino 7/5/2013 22:42'!
canBeRoot
	^ true! !

!ProjectLanding class methodsFor: 'testing' stamp: 'BrianDentino 7/16/2013 02:49'!
initialize 
	"ProjectLanding initialize"
	
	(WAAdmin register: self asApplicationAt: 'landing')
		addLibrary: JQDeploymentLibrary;
		addLibrary: JQUiDeploymentLibrary;
		addLibrary: JQCupertinoTheme.
	tables := Dictionary new.! !

!ProjectLanding class methodsFor: 'testing' stamp: 'BrianDentino 7/12/2013 01:00'!
tables
	^tables.! !

!ProjectLanding methodsFor: 'setup' stamp: 'lb 7/19/2013 22:54'!
createNewTable
	| id |
	id := self request: 'Enter a Table ID:'.
	(ProjectLanding tables) at: id put: (TexasHoldEm id: id UI: 'seaside').
	self joinTable: id.! !

!ProjectLanding methodsFor: 'initialization' stamp: 'BrianDentino 7/12/2013 00:57'!
initialize 
	super initialize.! !

!ProjectLanding methodsFor: 'setup' stamp: 'BrianDentino 7/12/2013 01:06'!
joinExistingTable
	| id |
	id := self requestTableID.
	self joinTable: id.
	! !

!ProjectLanding methodsFor: 'setup' stamp: 'BrianDentino 7/15/2013 23:20'!
joinTable: tableID
	self call: (PokerPlayerView tableID: tableID).
	! !

!ProjectLanding methodsFor: 'rendering' stamp: 'BrianDentino 7/14/2013 14:49'!
renderContentOn: html
	html heading: 'Welcome to Smalltalk Casino!!'.
	html anchor
		callback: [ self createNewTable. ];
		with: 'Start a table'.
	html space.
	html anchor
		callback: [ self joinExistingTable. ];
		with: 'Join a table'.! !

!ProjectLanding methodsFor: 'setup' stamp: 'BrianDentino 7/6/2013 02:09'!
requestTableID
	| id |
	id := self request: 'Enter a Table ID:'.
	^id! !

!ProjectLanding methodsFor: 'as yet unclassified' stamp: 'BrianDentino 7/5/2013 22:46'!
states
	^ Array with: self.! !

WAComponent subclass: #WebsocketHandlerComponent
	instanceVariableNames: 'wsAddress id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Seaside'!

!WebsocketHandlerComponent commentStamp: '<historical>' prior: 0!
A WebsocketHandlerComponent is a simple Seaside component that renders javascript capable of establishing a websocket connection between the rendering browser and a websocket server at the address held in wsAddress.

Along with connection logic, it renders a simple message handler that the browser will run. This handler expects JSON encoded messages with an 'id' property and a 'content' property. The message handler will replace the content of the DOM element identified by 'id' with the text of the 'content' property.

Instance Variables
	wsAddress:		<string>

wsAddress
	- string representation of the websocket address
	i.e. 'ws://10.0.0.3:8888/client'
!

!WebsocketHandlerComponent class methodsFor: 'initialize' stamp: 'BrianDentino 7/11/2013 19:51'!
wsAddress: address
	^ (self new) wsAddress: address.! !

!WebsocketHandlerComponent methodsFor: 'accessing' stamp: 'BrianDentino 7/12/2013 20:00'!
id
	^id! !

!WebsocketHandlerComponent methodsFor: 'accessing' stamp: 'BrianDentino 7/12/2013 20:00'!
initialize
	super initialize.
	id := 'websocket'! !

!WebsocketHandlerComponent methodsFor: 'accessing' stamp: 'BrianDentino 7/17/2013 23:53'!
renderContentOn: html

	html script: '	
	var conn = {};
	conn = new WebSocket("', wsAddress, '");
 
	conn.onmessage = function (event) {
		console.log(event.data);
		var json = event.data;
		var obj = JSON.parse(json); /** TODOhow to get jquery to work?? */
		
		console.log(obj.id);
		console.log(obj.content);
		
		var elt = document.getElementById(obj.id);
		elt.innerHTML = obj.content;
		updatePlayers();

    	};
    
    	conn.onclose = function (event) {
		console.log("connection closed")
    	};
	
'! !

!WebsocketHandlerComponent methodsFor: 'accessing' stamp: 'BrianDentino 7/11/2013 19:52'!
wsAddress
	^wsAddress! !

!WebsocketHandlerComponent methodsFor: 'accessing' stamp: 'BrianDentino 7/11/2013 19:52'!
wsAddress: address
	wsAddress := address.! !

Object subclass: #BrowserWebsocket
	instanceVariableNames: 'delegate websocket lock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Websocket'!

!BrowserWebsocket commentStamp: '<historical>' prior: 0!
A BrowserWebsocket is an abstraction over the websocket connection between a client PokerPlayer and the browser rendering it.

Instance Variables
	delegate:		<Object>

delegate
	- xxxxx
!

!BrowserWebsocket class methodsFor: 'initialize' stamp: 'BrianDentino 7/11/2013 22:20'!
delegate: aDelegate 
	^(self new delegate: aDelegate).! !

!BrowserWebsocket methodsFor: 'initialize-release' stamp: 'BrianDentino 7/11/2013 22:20'!
delegate: aDelegate
	delegate := aDelegate.! !

!BrowserWebsocket methodsFor: 'initialize-release' stamp: 'BrianDentino 7/12/2013 00:05'!
initialize
	lock := Mutex new.! !

!BrowserWebsocket methodsFor: 'initialize-release' stamp: 'lb 7/12/2013 16:25'!
sendMessage: aMessage
	websocket isNil ifTrue: [ Transcript show: 'browser websocket not ready yet...' ]
		ifFalse: [ websocket sendMessage: aMessage ].! !

!BrowserWebsocket methodsFor: 'initialize-release' stamp: 'lb 7/18/2013 01:42'!
value: aWebsocket 
	websocket := aWebsocket.
	
	lock critical: [ delegate 
		onMessage: ((WebsocketMessage id: 'joining' content:  (delegate playerID)) json) 
		from: self. ].
		
	[ websocket runWith: [ :message |
		websocket log info: [
			 'PokerPlayer received message from browser: ', message printString. 
		].
		lock critical: [ delegate onMessage: message from: self. ].
	]. ]
	on: ConnectionClosed 
	do: [ 
		websocket log debug: '**Connection close, cleaning up'.
		delegate close.
	].! !

!BrowserWebsocket methodsFor: 'as yet unclassified' stamp: 'lb 7/12/2013 13:42'!
websocket
^ websocket! !

Object subclass: #PokerTable
	instanceVariableNames: 'id lock players game waitingForResponse'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Websocket'!

!PokerTable commentStamp: '<historical>' prior: 0!
A PokerTable is a container for clients playing poker together. Each client is represented by a websocket. When the game state is updated, the table broadcasts updates to each of the clients.

Instance Variables
!

!PokerTable class methodsFor: 'as yet unclassified' stamp: 'lb 7/19/2013 10:12'!
id: anIdentifier game: aGame
	^ PokerTable new id: anIdentifier game: aGame! !

!PokerTable methodsFor: 'accessing' stamp: 'lb 7/19/2013 16:26'!
addPlayer: aPlayer at: aWebsocket
Transcript show: '**********************************   ', aPlayer name. 
	players at: aPlayer name put: aWebsocket.! !

!PokerTable methodsFor: 'websocket' stamp: 'lb 7/19/2013 16:27'!
broadcast: aMessage
	"Send message to all connected clients"

	lock critical: [ 
		players keysAndValuesDo: [  :playerID :ws |
			[ ws sendMessage: aMessage ]
				on: NetworkError
				do: [  :exception | 
					ws log debug: [ exception printString, ' NetworkError in broadcast...ignoring' ] ] ] ]! !

!PokerTable methodsFor: 'websocket' stamp: 'lb 7/19/2013 15:01'!
broadcastTableUpdate
	| content msg tableModel |

	content := WARenderCanvas builder render: tableModel.	
	msg := (WebsocketMessage id: 'pokerTable' content: content) json.
	self broadcast: msg.! !

!PokerTable methodsFor: 'protocol' stamp: 'lb 6/13/2013 21:50'!
clear
	players keysDo: [ :playerID | players removeKey: playerID.  ] ! !

!PokerTable methodsFor: 'accessing' stamp: 'lb 7/17/2013 21:58'!
gameFlowMessage: aString
	self halt.! !

!PokerTable methodsFor: 'accessing' stamp: 'lb 7/19/2013 23:18'!
getPlayerAction: aPlayer

	| msg |

	msg := (WebsocketMessage id: 'bet' content: aPlayer name) json.
	self broadcast: msg.
	
	waitingForResponse whileTrue: [ (Delay forMilliseconds: 500) wait. ]! !

!PokerTable methodsFor: 'accessing' stamp: 'BrianDentino 7/6/2013 18:50'!
id
	^id! !

!PokerTable methodsFor: 'accessing' stamp: 'lb 7/19/2013 10:11'!
id: anIdentifier game: aGame
	game := aGame.
	id := anIdentifier.
	ZnServer default delegate map: id to: (ZnWebSocketDelegate map: id to: self).! !

!PokerTable methodsFor: 'initialize-release' stamp: 'lb 7/19/2013 23:16'!
initialize 
	players := Dictionary new.
	lock := Mutex new.! !

!PokerTable methodsFor: 'accessing' stamp: 'lb 7/19/2013 21:18'!
lastPlayerAction: aPlayerAction

	| content msg |

	content := WARenderCanvas builder render: aPlayerAction.	
	msg := (WebsocketMessage id: 'lastPlayerAction' content: content) json.
	self broadcast: msg.
	
	"self broadcast: (WebsocketMessage id: 'updatePlayerAction' content: aPlayerAction) json."! !

!PokerTable methodsFor: 'accessing' stamp: 'lb 6/13/2013 21:48'!
players
	^ players! !

!PokerTable methodsFor: 'accessing' stamp: 'lb 7/19/2013 15:47'!
refreshCommunity: aCommunity

	| content msg |

	content := WARenderCanvas builder render: aCommunity.	
	msg := (WebsocketMessage id: 'community' content: content) json.
	self broadcast: msg.! !

!PokerTable methodsFor: 'accessing' stamp: 'lb 7/19/2013 16:36'!
refreshHands: aPlayerList

	| content msg |
	
	aPlayerList do: [ :player |
		content := WARenderCanvas builder render: player hand.	
		msg := (WebsocketMessage id: 'hand' content: content) json.
		(players at: player name) sendMessage: msg.
	].

	! !

!PokerTable methodsFor: 'accessing' stamp: 'lb 7/19/2013 15:33'!
refreshPlayerList: aPlayerList

	| content msg |

	content := WARenderCanvas builder render: aPlayerList.	
	msg := (WebsocketMessage id: 'players' content: content) json.
	self broadcast: msg.! !

!PokerTable methodsFor: 'protocol' stamp: 'lb 6/13/2013 21:49'!
register: clientWebSocket
	Transcript show: 'shouldn''t be calling register...'
"	lock critical: [ 
		players at: id add: clientWebSocket ]."! !

!PokerTable methodsFor: 'protocol' stamp: 'lb 6/13/2013 21:48'!
unregister: clientWebSocket
	lock critical: [ 
		players keysAndValuesDo: [ :playerID :ws | ws = clientWebSocket ifTrue: [ players removeKey: playerID.  ] ]  ] .
	(players size = 0) ifTrue: [ProjectLanding tables removeKey: id]! !

!PokerTable methodsFor: 'protocol' stamp: 'lb 7/19/2013 10:16'!
value: websocket

	| msg handler |

	[ "self register: websocket."
		websocket runWith: [ :message |
			
			websocket log info: [ 'PokerTable received message: ', message printString ].

			msg := WebsocketMessage json: message.

			Transcript show: '*******'; show: msg id; show: msg content.
			
			handler := WebsocketMessageHandlerFactory getHandlerFor: msg. 
			
			handler handleMessage: msg from: websocket for: game.
			
		].
			
	]
	on: ConnectionClosed 
	do: [ 
		websocket log debug: '!!!!Connection close, cleaning up'.
		self unregister: websocket ].! !

Object subclass: #WebsocketMessage
	instanceVariableNames: 'id content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Websocket'!

!WebsocketMessage commentStamp: '<historical>' prior: 0!
A WebsocketMessage is an object intended to represent a websocket message. It contains an id, so the receiver can identify the type of message, as well as content.

Instance Variables
	content:		<string>
	id:		<string>

content
	- String intended to be interpreted as the content of the message

id
	- String intented to be interpreted as the message identifier.
!

!WebsocketMessage class methodsFor: 'creation' stamp: 'lb 7/13/2013 14:05'!
id: aStringId content: aStringContent 
	^ (self new id: aStringId) content: aStringContent.! !

!WebsocketMessage class methodsFor: 'as yet unclassified' stamp: 'lb 7/13/2013 15:08'!
json: aJsonString

	 |obj |
	
	obj := Json readFrom: (ReadStream on: aJsonString).

	^ WebsocketMessage id: obj id content: obj content! !

!WebsocketMessage methodsFor: 'accessing' stamp: 'BrianDentino 7/12/2013 19:56'!
content
	^content.! !

!WebsocketMessage methodsFor: 'accessing' stamp: 'BrianDentino 7/18/2013 03:23'!
content: aString 
	content := aString.! !

!WebsocketMessage methodsFor: 'accessing' stamp: 'BrianDentino 7/12/2013 19:56'!
id
	^id.! !

!WebsocketMessage methodsFor: 'accessing' stamp: 'lb 7/13/2013 14:05'!
id: aString 
	id := aString.! !

!WebsocketMessage methodsFor: 'accessing' stamp: 'lb 7/13/2013 15:05'!
json

	^ Json render: self! !

!WebsocketMessage methodsFor: 'as yet unclassified' stamp: 'lb 7/18/2013 00:47'!
jsonWriteOn: aWriteStream
	
	aWriteStream nextPutAll: '{ "id" : '.
	aWriteStream nextPutAll: (Json render: id).
	aWriteStream nextPutAll: ', "content" : '.
	"content isDictionary ifTrue: [ aWriteStream nextPutAll: '{' ]."
	aWriteStream nextPutAll: (Json render: content).
	"content isDictionary ifTrue: [ aWriteStream nextPutAll: '}' ]."
	aWriteStream nextPutAll: ' }'.
	^ aWriteStream contents! !

Object subclass: #WebsocketMessageHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Websocket'!

WebsocketMessageHandler subclass: #ActivateMessageHandlerTESTING
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Websocket'!

!ActivateMessageHandlerTESTING methodsFor: 'as yet unclassified' stamp: 'lb 7/19/2013 10:14'!
handleMessage: aWebsocketMessage from: aWebsocket for: aGame
	" just for testing...this message simulates a user action that makes the requesting player active "
	aGame activatePlayer: aWebsocketMessage content.! !

WebsocketMessageHandler subclass: #CommMessageHandlerTESTING
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Websocket'!

!CommMessageHandlerTESTING methodsFor: 'as yet unclassified' stamp: 'lb 7/19/2013 10:58'!
handleMessage: aWebsocketMessage from: aWebsocket for: aGame
	" just for testing... the addCommunityCard function should be called by the game not ever the browser message handlers "
	aGame communityCardTesting! !

WebsocketMessageHandler subclass: #DrawMessageHandlerTESTING
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Websocket'!

!DrawMessageHandlerTESTING methodsFor: 'as yet unclassified' stamp: 'lb 7/19/2013 15:56'!
handleMessage: aWebsocketMessage from: aWebsocket for: aGame
	" just for testing... the dealCard function should be called by the game not ever the browser message handlers "
	aGame handTesting! !

WebsocketMessageHandler subclass: #JoinMessageHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Websocket'!

!JoinMessageHandler methodsFor: 'as yet unclassified' stamp: 'lb 7/19/2013 10:15'!
handleMessage: aWebsocketMessage from: aWebsocket for: aGame
	
	"adds a player to the dictionary of players... even if a websocket connects the player doesn't officially
	join the game until a Joining message is sent (because we need the playerID)"
	
	"add the player to the game... then somewhere we need to add them to the websocket list as well ** so need websocket here ** "
	aGame addPlayer: aWebsocketMessage content at: aWebsocket.! !

WebsocketMessageHandler subclass: #PlayerActionMessageHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Websocket'!

!PlayerActionMessageHandler methodsFor: 'as yet unclassified' stamp: 'lb 7/19/2013 10:15'!
handleMessage: aWebsocketMessage from: aWebsocket for: aGame "shouldn't need these other 2 args i don't think(?)... just for now"
	
	| jsonObj action |

	jsonObj := aWebsocketMessage content.
	
	action := THPlayerActionFactory new create: jsonObj.
	
	" do something with the action... "
	
	aGame updatePlayerAction: action. " then this line would be after whatever you do with the action... "! !

!WebsocketMessageHandler methodsFor: 'as yet unclassified' stamp: 'lb 7/19/2013 10:14'!
handleMessage: aWebsocketMessage from: aWebsocket for: aGame
	self subclassResponsibility! !

Object subclass: #WebsocketMessageHandlerFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-Project-Websocket'!

!WebsocketMessageHandlerFactory class methodsFor: 'as yet unclassified' stamp: 'lb 7/19/2013 11:09'!
getHandlerFor: msg
				
	(msg id = 'playerAction')
		ifTrue: [ ^ PlayerActionMessageHandler new ].	
			
	(msg id = 'joining') 
		ifTrue: [ ^ JoinMessageHandler new. ].	
					
	(msg id = #handTesting) 
		ifTrue: [ ^ DrawMessageHandlerTESTING new. ].
	
	(msg id = #commTesting)
		ifTrue: [  ^ CommMessageHandlerTESTING new ].		
				
	(msg id = 'activate')
		ifTrue: [ ^ ActivateMessageHandlerTESTING new ].
		
	"Should never reach this point"
	self error: 'Unknown Message ID: ' , msg id! !
PokerPlayerView initialize!
ProjectLanding initialize!
